from typing import List, Union

import psutil


class ProcessService:
    def find_pid(self, process_name: str) -> Union[str, int]:
        for process in psutil.process_iter():
            try:
                if process.name() == process_name:
                    return process.pid
            except OSError:
                continue

        return -1

    def process_is_running(self, process_name: str) -> bool:
        return self.find_pid(process_name) != -1

    def any_process_is_running(self, process_names: List[str]) -> bool:
        for process in psutil.process_iter():
            try:
                if process.name() in process_names:
                    return True
            except OSError:
                continue

        return False



import argparse
import os

import confuse

from bgm.music_player import MusicPlayer
from bgm.music_state_machine import MusicStateMachine
from bgm.process_service import ProcessService


def main() -> None:
    parser = argparse.ArgumentParser(description="Parse arguments for debugging purposes")
    parser.add_argument("-es", nargs="?", help="the process name for emulationstation")
    parser.add_argument(
        "-names",
        metavar="ENames",
        type=str,
        nargs="*",
        help="the emulator names you want to consider",
    )

    args = parser.parse_args()
    config = confuse.Configuration("esbgm", __name__)
    config.set_args(args, dots=True)
    print("configuration directory is", config.config_dir())
    userconfig_path = os.path.join(config.config_dir(), confuse.CONFIG_FILENAME)
    if not os.path.exists(userconfig_path):
        print(f"config file not found, dumping defaults at {userconfig_path}")
        yaml = config.dump()
        with open(userconfig_path, "w") as fs:
            fs.write(yaml)

    MusicStateMachine(ProcessService(), MusicPlayer(), config).run()


if __name__ == "__main__":
    main()



import os
from typing import Dict, List, Optional, Type, Union

import yaml
from deepmerge import Merger
from pydantic import BaseModel, ValidationError

default_merger = Merger(
    # pass in a list of tuple, with the
    # strategies you are looking to apply
    # to each type.
    [(dict, ["merge"])],
    # next, choose the fallback strategies,
    # applied to all other types:
    ["override"],
    # finally, choose the strategies in
    # the case where the types conflict:
    ["override"],
)


class _Optional(str):
    """
    Wrap a file path with this class to mark it as optional.

    Optional paths don't raise an :class:`IOError` if file is not found.
    """


class ESBgmConfigSchema(BaseModel):
    startdelay: int
    musicdir: str
    restart: bool
    startsong: Optional[str]
    mainprocess: str
    emulation_names: List[str]


def merge(*dicts: Dict) -> Dict:
    if len(dicts) < 2:
        raise Exception("Need at least two dicts to merge")

    base: Dict = {}
    for dict in dicts:
        default_merger.merge(base, dict)

    return base


def load_registry(base_dir: str, setting_paths: List[Union[str, _Optional]], schema: Type[BaseModel]) -> Dict:
    registry_raw: Dict = {}
    for path_str in setting_paths:
        path = os.path.join(base_dir, path_str)
        if os.path.exists(path):
            with open(path) as fs:
                registry_raw = merge(registry_raw, yaml.safe_load(fs))
        elif not isinstance(path_str, _Optional):
            raise OSError(f"No such file: {path_str}")

    try:
        registry = schema(**registry_raw)
    except ValidationError as e:
        raise ValueError(e)

    return registry.dict()
